require 'settings'
require 'moveable_game_object'
require 'items/bomb'
require 'items/explosion'
require 'secondary_weapons/snail_weapon'

class Player < MovableGameObject
  attr_accessor :number, :explosive_force, :max_bombs, :speed, :bombs, :lives,
                :killed_by, :kills, :deaths,
                :secondary_weapon, :keys, :moving_to
  
  def initialize(number, sprite_number, keys = nil)
    @number = number
    
    super V(0, 0), Sprite.new("player#{sprite_number}.png")
    
    @keys = keys
    @direction = V(0, 0)
    
    @kills     = 0
    @deaths    = 0
  end
  
  def prepare_for_game(start_point, reset_specs = true)
    @pos           = start_point * 40
    @moving_to     = nil
    @direction     = V(0, 0)
    @last_movement = nil
    @bombs         = []
    @hurt_by       = []
    @killed_by     = nil
    
    if reset_specs
      @speed           = Settings.gameplay.initial_speed
      @explosive_force = Settings.gameplay.initial_explosive_force
      @max_bombs       = Settings.gameplay.initial_max_bombs
      @lives           = Settings.gameplay.initial_lives
    end
  end
  
  def update(dtime)
    @secondary_weapon.trigger if @using_secondary_weapon && !@secondary_weapon.nil?
    
    move(dtime)
    
    colliding_fields = Game.current_level.find_collisions(@pos, 10)
    colliding_fields.each do |field|
      field.items.each_with_index do |item, i|
        if item.class == Explosion && !@hurt_by.include?(item.explosion_id)
          @hurt_by += [item.explosion_id]
          @lives -= 1
          if @lives == 0
            @killed_by        = item.player
            @deaths          += 1
            @killed_by.kills += 1 unless @killed_by.nil? || @killed_by == self
          end
        end
      end
    end
    
    if @planting_bombs
      field = Game.current_level[Game.current_level.field_at(@pos)]
      if field.distance_from_center(@pos) < 10 && field.can_plant_bomb? && @max_bombs > @bombs.size
        Bomb.new(field, self)
      end
    end
  end
  
  def move(dtime)
    dt = [15, dtime].max
    dtime -= dt
    
    current_field = Game.current_level.field_at(@pos)
    if @moving_to.nil?
      if !@last_movement.nil? && @last_movement.x.abs > @last_movement.y.abs
        if @direction.y != 0
          move_to = current_field + V(0, @direction.y) if Game.current_level.in_level?(current_field + V(0, @direction.y))
        end
        if @direction.x != 0 && (move_to.nil? || !Game.current_level[move_to].walkable?)
          move_to = current_field + V(@direction.x, 0) if Game.current_level.in_level?(current_field + V(@direction.x, 0))
        end
      else
        if @direction.x != 0
          move_to = current_field + V(@direction.x, 0) if Game.current_level.in_level?(current_field + V(@direction.x, 0))
        end
        if @direction.y != 0 && (move_to.nil? || !Game.current_level[move_to].walkable?)
          move_to = current_field + V(0, @direction.y) if Game.current_level.in_level?(current_field + V(0, @direction.y))
        end
      end
    end
  
    if !move_to.nil? && Game.current_level[move_to].walkable?
      @moving_to = move_to
    end
  
    unless @moving_to.nil?
      offset = (@moving_to*40 - @pos).normalize * @speed * dt
      @pos += offset
      @last_movement = offset
      if (@moving_to*40 - @pos).abs < offset.abs
        @moving_to = nil
        self.center_on_field
      end
    end
  
    Game.current_level[current_field].walked_over(self)
  end
  
  def center_on_field
    fs = 40
    @pos = Game.current_level.field_at(@pos) * fs
  end
  
  def handle_event(event)
    unless @keys.nil?
      case event
        when Rubygame::Events::KeyPressed
          if event.key == @keys[:right]
            @direction += V(1, 0)
          elsif event.key == @keys[:left]
            @direction += V(-1, 0)
          elsif event.key == @keys[:up]
            @direction += V(0, -1)
          elsif event.key == @keys[:down]
            @direction += V(0, 1)
          elsif event.key == @keys[:plant_bomb]
            @planting_bombs = true
          elsif event.key == @keys[:secondary_weapon]
            @using_secondary_weapon = true
          end
        when Rubygame::Events::KeyReleased
          if event.key == @keys[:right]
            @direction -= V(1, 0)
          elsif event.key == @keys[:left]
            @direction -= V(-1, 0)
          elsif event.key == @keys[:up]
            @direction -= V(0, -1)
          elsif event.key == @keys[:down]
            @direction -= V(0, 1)
          elsif event.key == @keys[:plant_bomb]
            @planting_bombs = false
          elsif event.key == @keys[:secondary_weapon]
            @using_secondary_weapon = false
          end
      end
    end
  end
  
  def draw_stats(srf, pos)
    @sprite.draw srf, pos + V(10, 10)
    
    # Kills
    t = Game.font.render(@kills.to_s, false, [50, 200, 50])
    t.blit srf, (pos + V(110 - t.w/2.0, 30 - t.h/2.0)).to_a
    # Deaths
    t = Game.font.render(@deaths.to_s, false, [200, 50, 50])
    t.blit srf, (pos + V(180 - t.w/2.0, 30 - t.h/2.0)).to_a
    
    Game.life_sprite.draw srf, pos + V(10, 60)
    Game.bomb_sprite.draw srf, pos + V(70, 60)
    Game.explosion_sprite.draw srf, pos + V(130, 60)
    
    t = Game.font.render("#{@lives}", false, [255, 255, 255])
    t.blit srf, (pos + V(30 - t.width/2, 110)).to_a
    
    t = Game.font.render("#{@max_bombs}", false, [255, 255, 255])
    t.blit srf, (pos + V(90 - t.width/2, 110)).to_a
    
    t = Game.font.render("#{@explosive_force}", false, [255, 255, 255])
    t.blit srf, (pos + V(150 - t.width/2, 110)).to_a
  end
  
  def draw_selection_menu(srf, pos, joined)
    @sprite.draw srf, pos + V(10, 10)
    
    # Kills
    t = Game.font.render(@kills.to_s, false, [50, 200, 50])
    t.blit srf, (pos + V(110 - t.w/2.0, 30 - t.h/2.0)).to_a
    # Deaths
    t = Game.font.render(@deaths.to_s, false, [200, 50, 50])
    t.blit srf, (pos + V(180 - t.w/2.0, 30 - t.h/2.0)).to_a
    
    if joined
      #t = Game.font.render("Joined!", false, [255, 255, 255])
      #t.blit srf, (pos + V(70, 20)).to_a
      t = Game.font.render(@keys[:up].to_s, false, [255, 255, 255])
      t.blit srf, (pos + V(110 - t.w/2.0, 70 - t.h/2.0)).to_a
      t = Game.font.render(@keys[:down].to_s, false, [255, 255, 255])
      t.blit srf, (pos + V(110 - t.w/2.0, 130 - t.h/2.0)).to_a
      t = Game.font.render(@keys[:left].to_s, false, [255, 255, 255])
      t.blit srf, (pos + V(50 - t.w/2.0, 100 - t.h/2.0)).to_a
      t = Game.font.render(@keys[:right].to_s, false, [255, 255, 255])
      t.blit srf, (pos + V(170 - t.w/2.0, 100 - t.h/2.0)).to_a
    else
      "Press #{@keys[:plant_bomb]} to join!".split("\n").each_with_index do |s, i|
        t = Game.font.render(s, false, [255, 255, 255])
        t.blit srf, (pos + V(10, 60 + i*25)).to_a
      end
    end
  end
end
